@@ -9,7 +9,7 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 #  --------------------------------------------------------------------------
-
+import ast
 import base64
 import csv
 import glob
@@ -56,19 +56,35 @@ class NoCredentialsToCreateSpace(Error):
 
 
 #
-# We patch APIClient._params with this function instead
+# The WML API offers a way to choose which entities are returned when asking
+# for job details. Unfortunately, that parameter is not surfaced in the WML
+# Python API, and we have to patch the code in order to send the value we want.
 #
-the_filter = None
-the_old_params = None
+# APIClient._params() is the function that creates the parameters for the REST
+# call.  We replace it with our own function '_new_params' that (1) calls the
+# original function and adds the filter we want, if we want one.
+#
+
+# The function will not be called with any parameter. So the filter we want to
+# use, if any, must be set in a global variable.
+_the_filter = None
+# Backup of the original function, so that we can call it and restore it later.
+_the_old_params = None
 
 
-def new_params():
-    global the_old_params
-    global the_filter
-    result = the_old_params()
-    if the_filter:
-        # Beware: the parameter list must not have spaces!
-        result['include'] = the_filter
+def _new_params():
+    """Our new function to build a parameter list for the REST call.
+
+    Called by the instance of APIClient itself."""
+    global _the_old_params
+    global _the_filter
+    # Use the original code and get its output
+    result = _the_old_params()
+    # Add the filter, if one is required
+    if _the_filter:
+        # The filter doesn't work correctly if it contains spaces
+        assert(_the_filter.find(' ') == -1)
+        result['include'] = _the_filter
     return result
 
 
@@ -114,11 +130,11 @@ class _CredentialsProvider:
 
     def check_credentials(self, wml_cred_str):
         assert type(wml_cred_str) is str
-        if len(wml_cred_str) == 0:
+        if not wml_cred_str:
             self._logger.error('WML credentials must not be an empty string.')
             self.usage()
             raise InvalidCredentials
-        wml_credentials = eval(wml_cred_str)
+        wml_credentials = ast.literal_eval(wml_cred_str)
         assert type(wml_credentials) is dict
         assert (self.APIKEY in wml_credentials or self.TOKEN in wml_credentials)
         if self.APIKEY in wml_credentials:
@@ -128,7 +144,7 @@ class _CredentialsProvider:
         assert self.URL in wml_credentials
         assert type(wml_credentials[self.URL]) is str
         url = wml_credentials[self.URL]
-        if len(url) == 0:
+        if not url:
             self._logger.error('URL must not be an empty string.')
             raise InvalidCredentials
         if url[-1] == '/':
@@ -165,7 +181,9 @@ class _CredentialsProvider:
 
 
 def version_is_greater(current, minimum):
-    """returns True is the current version string is greater or equal to the
+    """Compare two 'vv.nn.pp' versions.
+
+    Return True is the current version string is greater or equal to the
 minimum string.  Assumes that each string is of type vv.nn.pp, with vv, nn and
 pp being integers."""
     return version.parse(current) >= version.parse(minimum)
@@ -296,7 +314,7 @@ class DOWMLLib:
         return job_id
 
     def get_log(self, job_id):
-        """Extracts the engine log from the job
+        """Extract the engine log from the job.
 
         :param job_id: The id of the job to get the log from
         :return: The decoded log, or None
@@ -319,7 +337,7 @@ class DOWMLLib:
         return None
 
     def get_output(self, details, csv_as_dataframe=None, tabular_as_csv=False):
-        """"Extracts the outputs from the job
+        """"Extract the outputs from the job.
 
         :param details: The details of the job to get the output from
         :param csv_as_dataframe: Whether the content of a CSV file should be
@@ -410,15 +428,19 @@ class DOWMLLib:
 
     @staticmethod
     def client_get_job_details(client, job_id, with_filter=None):
-        global the_filter
-        global the_old_params
-        the_filter = with_filter
-        the_old_params = client._params
-        client._params = new_params
+        global _the_filter
+        global _the_old_params
+        # Save the filter in a global variable for our new function to find it
+        _the_filter = with_filter
+        # Save the pointer to the original code
+        _the_old_params = client._params
+        # and replace it with our new function
+        client._params = _new_params
         try:
             result = client.deployments.get_job_details(job_id)
         finally:
-            client._params = the_old_params
+            # Put back the original code
+            client._params = _the_old_params
         return result
 
     @staticmethod
@@ -462,7 +484,7 @@ class DOWMLLib:
                     self._logger.error('Exception raised while trying to delete the asset', exc_info=True)
 
     def delete_job(self, job_id, hard=False):
-        """ Delete the given job
+        """Delete the given job.
 
         :param job_id: the job to be deleted
         :param hard: if False, cancel the job. If true, delete it completely
@@ -475,7 +497,7 @@ class DOWMLLib:
         self._logger.debug('Done.')
 
     def decode_log(self, output):
-        """ Decode the log from DO4WML
+        """Decode the engine log coming from DO4WML.
 
         :param output: A base-64 encoded text with empty lines
         :return: The decoded text, without empty lines
@@ -488,7 +510,7 @@ class DOWMLLib:
 
     @staticmethod
     def remove_empty_lines(output):
-        """Remove empty lines from the log
+        """Remove empty lines from the log.
 
         :param output: The text to process
         :return: The text, with no empty lines
@@ -549,7 +571,7 @@ class DOWMLLib:
             time.sleep(delay)
 
     def wait_for_job_end(self, job_id, print_activity=False):
-        """Wait for the job to finish, return its status and details as a tuple"""
+        """Wait for the job to finish, return its status and details as a tuple."""
         client = self._get_or_make_client()
         delayer = DOWMLLib.ProgressiveDelay()
         while True:
@@ -579,7 +601,7 @@ class DOWMLLib:
 
     @staticmethod
     def get_file_as_data(path):
-        """Returns the base-64 encoded content of a file"""
+        """Return the base-64 encoded content of a file."""
         with open(path, 'rb') as f:
             data = f.read()
         data = base64.b64encode(data)
@@ -643,7 +665,7 @@ class DOWMLLib:
             return '?'
 
     def get_jobs(self):
-        """Return the list of tuples (status, id, ...) for all jobs in the deployment"""
+        """Return the list of tuples (status, id, ...) for all jobs in the deployment."""
         client = self._get_or_make_client()
         self._logger.debug('Getting job details...')
         job_details = client.deployments.get_job_details()
@@ -667,7 +689,7 @@ class DOWMLLib:
         return result
 
     def parse_paths(self, paths):
-        """Expand wildcards that may appear in the input assets list"""
+        """Expand wildcards that may appear in the input assets list."""
         self._logger.debug(f'Parsing input list: {paths}')
         globbed = []
         for path in paths.split():
@@ -714,7 +736,7 @@ class DOWMLLib:
             solve_payload[cdd_inputdata].append(input_data)
 
     def create_job(self, paths, deployment_id):
-        """Create a deployment job (aka a run) and return its id"""
+        """Create a deployment job (aka a run) and return its id."""
         client = self._get_or_make_client()
         cdd = client.deployments.DecisionOptimizationMetaNames
         assert(self.outputs == 'inline' or self.outputs == 'assets')
@@ -766,7 +788,7 @@ class DOWMLLib:
         return job_id
 
     def _get_deployment_id(self):
-        """Create deployment if doesn't exist already, return its id"""
+        """Create the deployment if doesn't exist already, return its id."""
         self._logger.debug('Getting deployments...')
         client = self._get_or_make_client()
         deployment_details = client.deployments.get_details()
@@ -807,7 +829,7 @@ class DOWMLLib:
         return deployment_id
 
     def _get_model_id(self):
-        """Create an empty model if one doesn't exist, return its id"""
+        """Create an empty model if one doesn't exist, return its id."""
         self._logger.debug('Getting models...')
         client = self._get_or_make_client()
         details = client.repository.get_details()
@@ -828,7 +850,7 @@ class DOWMLLib:
         return model_id
 
     def get_available_versions(self):
-        """Returns list of available DO versions on the platform"""
+        """Return the list of available DO versions on the platform."""
         client = self._get_or_make_client()
         target_version = "1.0.92"
         if not version_is_greater(client.version, target_version):
@@ -869,11 +891,11 @@ class DOWMLLib:
             os.remove(path)
         self._logger.debug('Model created.')
         model_id = client.repository.get_model_id(model_details)
-        self._logger.debug('Model id: {model_id}')
+        self._logger.debug(f'Model id: {model_id}')
         return model_id
 
     def _find_or_create_space(self):
-        """Find the Space to use from its name, create it if it doesn't exist"""
+        """Find the Space to use from its name, create it if it doesn't exist."""
         assert self._client
         client = self._client
         self._logger.debug('Fetching existing spaces...')
@@ -921,7 +943,7 @@ class DOWMLLib:
         return space_id
 
     def _get_asset_details(self):
-        """This function returns the list of all the data assets in the space"""
+        """Return the list of all the data assets in the space."""
         client = self._get_or_make_client()
         # This is the first version where data_assets.get_details() works
         assert(version_is_greater(client.version, "1.0.95.1"))
@@ -938,7 +960,7 @@ class DOWMLLib:
         return None
 
     def create_asset(self, path, basename):
-        """Create a data asset with the given name
+        """Create a data asset with the given name.
 
         A Watson Studio data asset is an entity that mimicks a file."""
         client = self._get_or_make_client()
@@ -946,7 +968,7 @@ class DOWMLLib:
         return asset_details['metadata']['guid']
 
     def delete_asset(self, uid):
-        """Delete an existing asset. Return True if ok, False if not"""
+        """Delete an existing asset. Return True if ok, False if not."""
         client = self._get_or_make_client()
         status = client.data_assets.delete(uid)
         return status == "SUCCESS"
